package com.gxzc.zen.orm

import com.gxzc.zen.orm.annotation.DynamicDataSource
import com.gxzc.zen.orm.contants.DSKey
import org.aopalliance.intercept.MethodInterceptor
import org.aopalliance.intercept.MethodInvocation
import java.lang.reflect.Method

/**
 *
 * @author NorthLan at 2018/1/31
 */
class DataSourceSwitchMethodInterceptor : MethodInterceptor {
    override fun invoke(invocation: MethodInvocation): Any {
        val packageName = invocation.`this`::class.java.`package`.name
        matchDataSource(invocation.method, packageName)
        return invocation.proceed()
    }

    private fun matchDataSource(method: Method, packageName: String) {
        DSKey.values()
                .filter { it.pkgName in packageName }
                .forEach { setDataSource(method, it.dsKey) }
    }

    /**
     * 设置数据源key
     */
    private fun setDataSource(method: Method, defaultKey: String) {
        val dynamicDataSource: DynamicDataSource? = method.getAnnotation(DynamicDataSource::class.java)
        DynamicMultipleDataSource.setDataSource(dynamicDataSource?.value ?: defaultKey)
    }
}